import os
import os.path
import subprocess
import stat

class ServiceConfig(object):
    def __init__(self, template_file,
                 name="friskby",
                 config_file="/etc/systemd/system/friskby.service"):
        with open(template_file, 'r') as fp:
            self.content = fp.read()

        if self.content.find("%(EXECUTABLE)s") < 0:
            raise ValueError("Missing %(EXECUTABLE)s in template")

        self.config_file = config_file
        self.name = name


    @classmethod
    def systemd(cls):
        return os.path.isdir("/etc/systemd/system")


    def save(self, executable):
        if not os.path.isfile(executable) or not os.access(executable, os.X_OK):
            raise OSError("Executable: %s was not usable" % executable)
        with open(self.config_file, "w") as f:
            f.write(self.content % {"EXECUTABLE" : os.path.abspath(executable)})

        mode = os.stat(self.config_file).st_mode
        mode |= stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH
        os.chmod(self.config_file, mode)


    def enable(self, start=True):
        subprocess.check_call(["systemctl", "daemon-reload"])
        subprocess.check_call(["systemctl", "enable", self.name])
        if start:
            subprocess.check_call(["systemctl", "start", self.name])



#!/usr/bin/env python
from __future__ import (print_function, absolute_import)

import sys
import os
import json
from traceback import format_exception

import friskby
from .friskby_dao import FriskbyDao
from .device_config import DeviceConfig
from .git_module import GitModule
from .os_release import sys_info

class FriskbyRunner(object):

    def __init__(self, root=None, config_file=None, var_path=None):
        if not root:
            self.root = os.path.abspath(os.path.join(os.path.dirname(__file__), "../"))
        self.config_file = config_file
        if not config_file:
            self.config_file = os.path.join(self.root, "etc/config.json")
        if not var_path:
            var_path = os.path.join(self.root, "var")

        self._sample_time = 10 * 60
        self._sleep_time = 0.5
        self._accuracy = 4 # round observation to fourth digit
        self._config = None
        self._sql_path = os.path.join(var_path, 'friskby.sql')
        self._dao = FriskbyDao(self._sql_path)

    def get_dao(self):
        return self._dao

    def install(self, config):
        git_module = GitModule(url=config.getRepoURL())
        git_module.checkout(config.getGitRef())
        git_module.runTests("tests/run_tests")
        git_module.install(self.root,
                           files=None,
                           directories=None) # TODO
        config.save(filename=self.config_file)


    def rollback(self, config):
        self.install(config)


    def restart(self, config):
        self.install(config)
        os.execl(__file__, __file__)

        raise Exception("Fatal error: os.execl() returned - trying to rollback")

    def update_client(self, config):
        new_config = config.downloadNew()
        if config.updateRequired(new_config):
            try:
                config.logMessage("Restarting client - new version:%s" % new_config.getGitRef())
                self.restart(new_config)
            except:
                exc_type, exc_value, exc_tb = sys.exc_info()
                tb_list = format_exception(exc_type, exc_value, exc_tb)
                config.logMessage("Restart failed - trying rollback", long_msg="".join(tb_list))
                self.rollback(config)
                config.logMessage("Rollback complete")


    def _handle_post_exception(self, err, exc_info):
        err_msg = '(unknown error for %s)' % str(type(err))
        try:
            err_msg = str(err)
        except Exception:
            pass
        exc_type, exc_value, exc_tb = exc_info
        tb_list = format_exception(exc_type, exc_value, exc_tb)
        log_payload = 'Exception caught: "%s".' % (err_msg)
        traceback = "".join(tb_list)
        try:
            self._config.logMessage(log_payload, long_msg=traceback)
        except:
            sys.stderr.write('Error submitting log message!')

    def run(self):
        self._config = DeviceConfig(self.config_file)
        long_msg = self.get_sys_info()
        self._config.logMessage("Starting up", long_msg=long_msg)
        self._config.postGitVersion()

        try:
            self.update_client(self._config)
        except Exception as err:
            self._handle_post_exception(err, sys.exc_info())
            raise

    @classmethod
    def get_sys_info(cls):
        info = ''
        try:
            info = sys_info()
            if info:
                info = json.dumps(info, indent=4, sort_keys=True)
        except Exception as err:
            info = 'Error getting sys_info: "%s".' % err
        return info



"""The friskby module
"""

from __future__ import absolute_import

from serial import SerialException
from .git_module import GitModule
from .device_config import DeviceConfig
from .friskby_dao import FriskbyDao
from .friskby_sampler import FriskbySampler
from .friskby_runner import FriskbyRunner
from .friskby_submitter import FriskbySubmitter
from .os_release import sys_info

# if _os_getenv("FRISKBY_TEST"):
try:
    from sds011 import SDS011
except ImportError as err:
    import sys
    sys.stderr.write('Missing SDS011 library: %s\n' % str(err))
    sys.stderr.write('Using mock library.\n')
    sys.stderr.flush()
    from .mock_sds011 import SDS011
except SerialException as err:
    import sys
    sys.stderr.write('Using mock SDS011 serial device: %s\n' % str(err))
    sys.stderr.write('Using mock library.\n')
    sys.stderr.flush()
    from .mock_sds011 import SDS011


from .service_config import ServiceConfig
from .ts import TS

__all__ = ['FriskbyDao', 'FriskbySampler', 'FriskbySubmitter', 'FriskbyRunner',
           'SDS011', 'ServiceConfig', 'TS', 'GitModule', 'DeviceConfig',
           'sys_info']

__version__ = '0.61.0'
